/*============================================================================
* PROJEKT:  
* MODUL:    Spinel.c
* PROCESOR: Z8F Encore 1601 
* VERZE:    1.0_DMA
* DATUM:    15.3.2004
* ===========================================================================
* Popis:  obsluha UARTu a zpracovani protokolu Spinel
* ===========================================================================
*
*              PaPouch s.r.o. - www.papouch.com
*               
* ===========================================================================
*/

#include <ez8.h>
#include "spinel_z8_dma.h"      	// header modulu spinel
#include "main_spinel_z8_dma.h"    	// header modulu main
 
#define	PREFIX	'*'

unsigned char spin_r_flags = FALSE;			// nulovat vsechny priznaky
#define		NOREC 		FALSE				// priznak pokud se neprijima
#define		RECEIVED	TRUE				// priznak pokud je prijem hotov
#define		REC_PREF	NOREC + 1			// priznak prijeti prefixu
#define		REC_FORM	REC_PREF + 1		// priznak prijeti formatu
#define		REC_LENH	REC_FORM + 1		// priznak prijeti Hbyte delky
#define		REC_LEN		REC_LENH + 1		// priznak prijeti Lbyte delky
#define		REC_ADDRH	REC_LEN + 1			// priznak prijeti adresy
#define		REC_ADDR	REC_ADDRH + 1		// priznak prijeti adresy
#define		REC_SIG		REC_ADDR + 1		// priznak prijeti podpisu


unsigned char 	spin_binary 	= FALSE;	// priznaky binarni zpravy
#define		BNA			0x80				// priznak cizi binarni zpravy
	
unsigned char 	spin_flags 	= FALSE;		// priznaky spinelu
#define		RX_BC			0x01			// priznak broadcastu
#define		RXTXSW			0x02			// priznak prepinani smeru komunikace - menitelne za behu programu
#define		RX_HN			0x40			// priznak Hnible prijimaneho byte
#define		TX_HN			0x80			// priznak Hnible vysilaneho byte



unsigned char spin_t_flags = FALSE;			// nulovat vsechny priznaky
/*#define		NOTX 		FALSE				// priznak pokud se nevysila
#define		TRANSMIT	NOTX + 1			// priznak vysilani
#define		TX_CS		TRANSMIT + 1		// priznak vyslani checksumu
#define		TX_CR		TX_CS + 1			// priznak vyslani CR
*/


unsigned char broadcast;	 				// prijem zpravy s broadcast adresou
unsigned char spin_addr = 0x01;				// moje adresa
unsigned char format;	 					// format
unsigned char txformat;	 					// format vysilanych dat
unsigned char ack;	 						// odpoved
unsigned char sig;	 						// podpis
unsigned int  rxlen;	 					// delka prijmu
unsigned int  txlen;	 					// delka vysilani
unsigned char rxchsum;						// kontrolni soucet prijmu
unsigned char txchsum;						// kontrolni soucet vysilani
unsigned char check_cs = TRUE;				// priznak kontroly checksumu
unsigned char txcount;						// citac vyslanych byte
unsigned char far *p_s_out;	
unsigned char commerr = 0x00; 				// citac chyb komunikace
unsigned char commspeed;					// index komunikacni rychlosti
unsigned char rectout;						// timeout prijmu
unsigned char rswaitc;						// citac prodlevy mezi RxTx
unsigned char volatile far u0_rx_h_buffer[RXBUFS];	// HW buffer prijimanych dat
unsigned char volatile far *p_rx_h_buff;			// ukazatel na HW buffer
unsigned char far recstr[MAXBLEN+1]; 		// prijaty retezec
unsigned char far hexstr[3];				// hex cislo v HESCII formatu
unsigned char far message[MAXBLEN+1];  		// vysilana zprava


unsigned char const name[] = "SpinelDemo; v0000.00; F65 97";
unsigned int const UARTspd[] = {0x188c, 0x0900, 0x0480, 0x0240,0x0120, 0x0090, 0x0048, 0x0024, 0x0012, 0x000c, 0x0006, 0x0003};	//konstanty pro 1200,2400,4800,9600,19200,38400,57600,115200, 230400




//---------------- Funkce pro HESCII format -----------------------------------
/*============================================================================
Funkce prevodu bin nible na hex

prevadi vstupni nible na ASCII znak 0 - 9, A - F

Vstup:  nible byte pro prevod
Vystup: ASCII HEX znak   
============================================================================*/
unsigned char b2hnconv(register unsigned char a)
 {
  a &= 0x0f;				// pro jistotu 
  if(a < 10)
   return(a + 48);			// znak 0 - 9
  else
   return(a + 55);			// znak A - F
 }

/*============================================================================
Funkce prevodu HESCII nible na bin

prevadi vstupni HESCI nible na bin

Vstup:  HESCII nible pro prevod
Vystup: bin, pri chybe vraci 0xff   
============================================================================*/
unsigned char hn2bconv(register unsigned char a)
 {
  if(a > '/' && a < ':')
   return(a - 48);			// znak 0 - 9
  else
   {
    if(a > '@' && a < 'G')
     return(a - 55);		// znak A - F
	else
	 return(0xff);
   }
 }

/*============================================================================
Funkce prevodu bin byte na HESCII

prevadi vstupni byte na ASCII znaky 00 - FF

Vstup:  byte pro prevod
Vystup: hexstr[]   
============================================================================*/
void bin2hex(unsigned char a)
 {
  hexstr[0] = b2hnconv(a >> 4);
  hexstr[1] = b2hnconv(a & 0x0f);
  hexstr[2] = '\0';
 }

/*============================================================================
Funkce prevodu HESCII byte na bin

prevadi vstupni HESCII byte na bin

Vstup:  H a L nible byte v HESCII
Vystup: bin byte   
============================================================================*/
unsigned char hex2bin(unsigned char a, unsigned char b)
 {
  a = hn2bconv(a);
  b = hn2bconv(b);
  return((a << 4) + (b & 0x0f));
 }

/*------------------------------------------------------------------
 DMA 0 Init

 inicializace DMA0 kanalu pro prijem na Uart0
-------------------------------------------------------------------*/
void init_DMA0(void)
 {
  register unsigned char b, c;
  register unsigned int d;

   p_rx_h_buff = &u0_rx_h_buffer[0];						// ukazatel na HW buffer
   d = (unsigned int)(&u0_rx_h_buffer[RXBUFS - 1]);	       	// koncova adresa bufferu
   D0END = (unsigned char)d;                        		// Lbyte adresy
   b = (unsigned char)((d >> 4) & 0xf0);               		// Lnible Hbyte 
   d = (unsigned int)&u0_rx_h_buffer[0]; 					// Lnible Hbyte pocatecni adresy
   c = (unsigned char)((d >> 8) & 0x0f);
   D0HIGH = (b | c);                              			// nastavit Hbyte obou adres
   D0START = (unsigned char)&u0_rx_h_buffer[0];            	// Lbyte pocatecni adresy
   D0IO = 0x40;                                 			// cist prijata data
   D0CTL = 0xe4;  		                               		// DMA na Rx UARTu0, Loop, NoInt
 }

/*------------------------------------------------------------------
 DMA 1 Init

 inicializace DMA1 kanalu pro vysilani na Uart0

 Vstup:   delka
-------------------------------------------------------------------*/
void init_DMA1(unsigned char l)
 {
  register unsigned char b, c;
  register unsigned int d;

   d = (unsigned int)(&message[0] + l);	            	// koncova adresa bufferu
   D1END = (unsigned char)d;                        	// Lbyte adresy
   b = (unsigned char)((d >> 4) & 0xf0);            	// Lnible Hbyte 
   d = (unsigned int)&message[0]; 						// Lnible Hbyte pocatecni adresy
   c = (unsigned char)((d >> 8) & 0x0f);
   D1HIGH = (b | c);                           			// nastavit Hbyte obou adres
   D1START = (unsigned char)&message[0];            	// Lbyte pocatecni adresy
   D1IO    = 0x40;                             			// cist data k odeslani
   D1CTL    = 0x94;                                		// DMA na Tx UARTu1, Int
 }

/*------------------------------------------------------------------
 Serial Init

 inicializace seriove komunikace
-------------------------------------------------------------------*/
void init_serial(void)
 {
  register unsigned int a;

  a = UARTspd[commspeed];			// nacist konstantu pro komunikacni rychlost
  U0BRH = a >> 8; 					// a nastavit ji
  U0BRL = (unsigned char)a;
  U0CTL0 = 0xc0;
  U0CTL1 = 0x02;
#if RS485							// pokud se jedna o simplexni komunikaci
  C_DE;								// prepnout na prijem
  spin_flags |= RXTXSW;				// a nastavit priznak obsluhy prepinani smeru
#else								// pokud jde o duplex
  spin_flags &= ~RXTXSW;			// zrusit priznak obsluhy prepinani smeru
#endif
  init_DMA0();
 }



//---------------- Funkce vypisu hlaseni --------------------------------------

/*============================================================================
  Create & Send Message 
  
  vytvori kompletni zpravu podle zadaneho formatu a zahaji vysilani
  
  vstup:	format
  			sig
			ack
  			delka zpravy (pocet byte)
        	retezec

  POZOR: Delka zpravy vcetne hlavicky a paticky muze byt max. 256 byte
============================================================================*/
void cs_sp_message(unsigned char f, unsigned char p, unsigned char a, unsigned char l, unsigned char *p_x) 
 {
  register unsigned char i, j, s = 0;
  register unsigned int c = 0;

  if(broadcast == TRUE)				// pokud se jedna o broadcast
   return;							// nic nevysilat
  while(spin_t_flags != NOTX)		// ceka na konec predchoziho vysilani - POZOR, WDT NENI OSETREN !!!
   {
    if(c++ >= 0xff00)
	 break;
   }

#if RS485							// pokud se jedna o simplexni komunikaci
  if(spin_flags & RXTXSW)			// pokud se jedna o rezim RS485
   {
    while(rswaitc != 0x00)			// ceka na konec prodlevy RxTx - POZOR, WDT NENI OSETREN !!!	
     asm ("nop");
   }
#endif

  switch(format)
   {
    case 'A':
	 message[0] = '*';				// prefix
	 message[1] = 'A';				// format
	 bin2hex(spin_addr);			// adresa
	 message[2] = hexstr[0];
	 message[3] = hexstr[1];
	 message[4] = p;				// podpis
	 bin2hex(a);					// ack
	 message[5] = hexstr[0];	
	 message[6] = hexstr[1];
     j = 7;
     for(i = 0; i < l; i++)
      {
       bin2hex(*p_x++);            		// byte
       message[j++] = hexstr[0];  		// Hnible
       message[j++] = hexstr[1];  		// Lnible
      }	
	 break;
	case 'a':
	 message[0] = '*';				// prefix
	 message[1] = 'a';				// format
//	 txlen += 2;					// k delce pricist i adresu  a sig
//	 p_x = (unsigned char *)&l;
	 message[2] = 0x00; //*p_x; 			// hbyte delky
//	 p_x++;
	 message[3] = l + 5; //*p_x; 			// lbyte delky
	 message[4] = spin_addr;		// adresa
	 message[5] = p;				// podpis
	 message[6] = a;				// ack
     j = 7;
     for(i = 0; i < l; i++)
      {
       message[j++] = *p_x++; 		// byte
      }
     for(i = 0; i < j; i++) 		// spocitat Check Sum 
      s += message[i];
     s = ~s;
     message[j++] = s;				// CheckSum
	 break;
	default:						// neznamy format
	 return;						// pouze navrat
	 break;
   }
  message[j] = '\r';         			// nakonec doplnit CR
  message[j+1] = '\0';					// jen pro poradek :-}

#if RS485								// pokud se jedna o simplexni komunikaci
  if(spin_flags & RXTXSW)				// pokud se jedna o rezim RS485
   S_DE;                     			// nahodit DE
#endif

  cled_count = 0;						// rozsvitit CLED
  spin_t_flags = TRANSMIT;         		// nastavit priznak vysilani
  init_DMA1(j);                  		// inicializovat DMA prenos
  U0TXD = message[0];
 }


/*============================================================================
  Send Name
  
  vysle adresu, jmeno zarizeni a verzi FW
============================================================================*/
void send_name(void)
 {
  register unsigned char i;
  register unsigned char n[sizeof(name)];

  for(i = 0; i < (sizeof(name) - 1); i++)
   n[i] = name[i];
  cs_sp_message(format, sig, ACK_OK, i, &n[0]); 
 }

//---------------- Funkce obsluhy seriove linky ------------------------------


/*=============================================================================
 UART Receive
 
 Vybere prijata data z HW bufferu, testuje prefix, adresu a konec prijmu
 na pocet znaku nebo CR
==============================================================================*/
unsigned char u0rx(void)
 {
  static unsigned char a;			// citac prijatych byte
  static unsigned int l;			// citac prijatych byte pro binarni format
  register unsigned char b, z;
  unsigned int volatile p;
 
  p = ((D0HIGH & 0x0f) << 8) + D0START;
  while((unsigned int)p_rx_h_buff != p)			// jsou nejaka data v bufferu?
   {
#if RS485										// pokud se jedna o simplexni komunikaci
    if(spin_flags & RXTXSW)						// pokud se jedna o rezim RS485
     rswaitc = RSWAIT;							// pri kazdem prijatem byte nastavit cekani Rx-Tx
#endif
	b = *p_rx_h_buff++;							// nacist byte
    if(p_rx_h_buff >= &u0_rx_h_buffer[RXBUFS])	// pretekl HW buffer?
     p_rx_h_buff = &u0_rx_h_buffer[0];			// nastavi ukazatel na zacatek
    if(b == '*' && spin_binary == FALSE)		// prijat prefix a nejedna se o binarni level?
     {
 	  spin_r_flags = REC_PREF;					// nulovani vsech docasnych priznaku a oznaceni priznaku
     }
    else
     {
      if(spin_r_flags >= REC_FORM && spin_r_flags < RECEIVED)	// prijem dalsich casti retezce za formatem ?
       {
	    switch(recstr[0])							// dalsi cinnost se ridi podle formatu
		 {
//     prijem formatu 65
          case 'A':					
		   if(spin_r_flags >= REC_ADDR)				// byla jiz prijata adresa? 
		    {
             cled_count = 0x00;						// s kazdym platnym byte rozsvitit LED
			 if(spin_r_flags >= REC_SIG)			// byl jiz prijat podpis?
			  {										// prijem instrukce
			   rectout = FALSE;						// nulovani timeoutu
			   if(b == 0x0d)						// test konce prijmu
         		{
				 recstr[3] = a - 4;					// ulozit delku dat
          		 recstr[a] = '\r';					// CR
          		 recstr[a+1] = 0x00;				// ukoncit retezec
                 spin_r_flags = RECEIVED;			// nastavit priznak prijateho retezce
				 return;
                }
			   else
			    {
                 if(spin_flags & RX_HN)				// byl jiz prijat vyssi nible?
			      {
			       if(b > '/' && b < ':')			// prevod z HESCII na bin
				    b -= 0x30;						// znak 0 - 9
                   else
				    {
                     if(b > '@' && b < 'G')
                      b -= 0x37;					// znak A - F
                     else
					  {
               	       b = 0;						// pri chybe zapsat 0
					   recstr[4] = 0;				// a na misto instrukce tez 0
					  }
                    }
			       recstr[a] += (b & 0x0f);			// sloucit H a L nible
                   spin_flags &= ~RX_HN;			// zrusit priznak prijmu vyssiho nible
				   a++;								// zvysit citac prijatych byte
				  }
			     else
			      {
				   if(b > '/' && b < ':')			// prevod z HESCII na bin
				    b -= 0x30;						// znak 0 - 9
                   else
				    {
                     if(b > '@' && b < 'G')
                      b -= 0x37;					// znak A - F
                     else
					  {
               	       b = 0;						// pri chybe zapsat 0
					   recstr[4] = 0;				// a na misto instrukce tez 0
					  }
                    }
			       recstr[a] = b << 4;				// ulozit nible
                   spin_flags |= RX_HN;				// nastavit priznak prijmu vyssiho nible
				  }
		         if(a == MAXBLEN + 1)				// neni jiz buffer plny 
		          a = MAXBLEN;
				}
			  }
			 else
			  {
			   recstr[2] = b;						// ulozit podpis
			   spin_r_flags = REC_SIG;				// a oznacit jeho ulozeni 
			   rectout = FALSE;						// nulovani timeoutu
			   a = 4;								// a nastavit citac pro dalsi prijem
			  }
			}
		   else
		    {
			 if(spin_r_flags == REC_ADDRH)
			  {
   		       if(b > '/' && b < ':')				// prevod z HESCII na bin
			    b -= 0x30;							// znak 0 - 9
               else
			    {
                 if(b > '@' && b < 'G')
                  b -= 0x37;						// znak A - F
                 else
                  b = 0;							// pri chybe zapsat jen 0
                }
			   b += recstr[1];						// sloucit H a L nible
  			   if(b == spin_addr || b > 0xfc)		// je to moje adresa nebo univerzal ci broadcast?
			    {
			     recstr[1] = b;						// ulozit adresu
			     spin_r_flags = REC_ADDR;			// oznacit prijem adresy
                 cled_count = 0x00;					// rozsvitit LED
			    }
     		   else									// jinak
                spin_r_flags = NOREC;				// zrusit priznak prijmu
 			  }
			 else
			  {
			   if(b > '/' && b < ':')				// prevod z HESCII na bin
			    b -= 0x30;							// znak 0 - 9
               else
			    {
                 if(b > '@' && b < 'G')
                  b -= 0x37;						// znak A - F
                 else
                  b = 0;							// pri chybe zapsat jen 0
                }
			   recstr[1] = b << 4;
			   spin_r_flags = REC_ADDRH;
			  }
			}										// a cekat na dalsi prefix
		   break;
/**/
//     prijem formatu 97
		  case 'a':
		   if(spin_r_flags >= REC_LEN)
		    {
	         if(spin_r_flags >= REC_ADDR)			// byla jiz prijata adresa? 
		      {
			   if(spin_binary == BNA)				// jedna se o cizi binarni zpravu?
			    {
				 l++;								// jen zvysit citac byte
     		     rectout = FALSE;					// nulovani timeoutu
				 if(rxlen == l)						// je uz konec zpravy?
				  {
				   spin_binary = FALSE;				// zrusit priznak binarni zpravy
                   spin_r_flags = NOREC;			// vynulovat priznaky prijmu
				  }
				}
			   else
			    {
                 cled_count = 0x00;					// s kazdym platnym byte rozsvitit LED
			     if(spin_r_flags >= REC_SIG)		// byl jiz prijat podpis?
			      {									// prijem instrukce
				   recstr[l] = b;					// ulozit byte
				   if(l < (rxlen - 1))				// ma se pocitat checksum?
				    rxchsum += b;					// checksum 
			       rectout = FALSE;					// nulovani timeoutu
				   if(rxlen == l)					// test konce prijmu
         		    {
				     recstr[3] = ((char)l) - 6;		// ulozit delku dat
          		     recstr[l] = '\0';				// ukoncit retezec
					 rxchsum = ~rxchsum;			// negovat checksum
					 if((rxchsum == recstr[l-1] || check_cs != TRUE)	// souhlasi checksum nebo se nema kontrolovat?
					    && (b == '\r'))				// a je posledni znak CR?
					  {
                       spin_r_flags = RECEIVED;		// nastavit priznak prijateho retezce
					   recstr[l-1] = '\r';			// na misto prijateho checksumu dosadit CR
   				       spin_binary = FALSE;			// zrusit priznak binarni zpravy
  				       return;		
					  }
					 else
					  {
					   commerr++;					// pri chybe zvysit citac
                       spin_r_flags = NOREC;		// vynulovat priznaky prijmu
					  }
   				     spin_binary = FALSE;			// zrusit priznak binarni zpravy
                	}
				   l++;								// zvysit citac prijatych byte
		         if(l == MAXBLEN + 1)				// neni jiz buffer plny 
                  l = MAXBLEN;
                  }
			    else
			     {
			      recstr[2] = b;					// ulozit podpis
				  rxchsum += b;						// checksum 
			      spin_r_flags = REC_SIG;			// oznacit prijeti 
			      rectout = FALSE;					// nulovani timeoutu
			      l = 4;							// a nastavit citac pro dalsi prijem
			     }
			    }
              }
		     else
		      {
  			   if((b == spin_addr) || (b > 0xfc))	// je to moje adresa nebo univerzal ci broadcast
			    {
			     recstr[1] = b; 					// ulozit adresu
				 rxchsum += b;						// checksum 
			     spin_r_flags = REC_ADDR;			// oznacit jeji prijem
                 cled_count = 0x00;					// rozsvitit LED
			    }
     		   else									// jinak
                spin_binary = BNA;					// nastavit priznak cizi adresy
 			  }
            }
		   else
		    {
			 if(spin_r_flags == REC_LENH)
			  {
			   rxlen += b;							// prevod delky na int
			   rxlen++;								// pro dalsi zpracovani je nutno zvysit delku zpravy
			   l = 0;								// nulovat pocet prijatych byte
			   rxchsum += b;						// checksum 
			   spin_r_flags = REC_LEN;				// oznacit jeji prijem
              }
			 else
			  {
			   rxlen = b * 0xff;
			   rxchsum += b;						// checksum 
			   spin_r_flags = REC_LENH;
			  }
			 }						
		   break;

//     prijem neznamych formatu
		  default:
		   if(spin_binary == TRUE)					// jedna se o binarni level?
		    {
			 if(spin_r_flags == REC_LEN)			// je uz znama delka retezce?
			  {
			   l++;
				if(rxlen == l)						// je uz konec zpravy?
				 {
				  spin_binary = FALSE;				// zrusit priznaky prijmu
 				  spin_r_flags = NOREC;				// zrusit priznak prijmu
				 }
			  }
			 else
			  {
			   if(spin_r_flags == REC_LENH)
			    {
			     rxlen += b;						// prevod delky na int
			     l = 0;								// nulovat pocet prijatych byte
			     spin_r_flags = REC_LEN;			// oznacit jeji prijem
                }
			   else
			    {
			     rxlen = b * 0xff;
			     spin_r_flags = REC_LENH;
			    }
			  }

			}
		   else										// pro ASCII format
		    {
 	         spin_r_flags = NOREC;					// zrusit priznak prijmu
			}
	       break;
		 }

       }
      else											// jeste neni format znamy
       {
        if(spin_r_flags == REC_PREF)				// prijem formatu?
	     {
          recstr[0] = b;							// ulozit prijaty byte oznacujici format
          spin_r_flags = REC_FORM;					// oznaceni prijmu
//          a = 1;									// nastavit citac prijatych byte
          rectout = FALSE;							// nulovani timeoutu
		  if(recstr[0] >= 'a' && recstr[0] <= 'z')	// jedna se o binarni format?
		   {
		    spin_binary = TRUE;						// oznacit binar
		    rxchsum = '*';							// kontrolni soucet, prefix
		    rxchsum += b;							// kontrolni soucet, format
		   }
         }
		else										// pokud nebyl prijat prefix
   	     commerr++;									// pricist chybu komunikace
       }
     }
   }
 }

/*------------------------------------------------------------------
 UART 0 TX Interrupt

 pouziva se pouze k ukonceni vysilani a prepnuti smeru
 v pripade, ze neni potreba prepinani smeru je mozno nastaveni priznaku
 konce vysilani napsat primo v obsluze interruptu DMA a interrupt od
 vysilani vubec nepovolovat a nevyuzivat
-------------------------------------------------------------------*/
#pragma interrupt
void isr_u0tx(void) 
 {
  static unsigned char volatile a = FALSE, b = FALSE;

  if(a == FALSE)						// 1. interupt se ignoruje
   a = TRUE;
  else									// pri 2. 
   {						
    while((U0STAT0 & 0x02) != 0x02);	// ceka se az bude skutecne prazdny shift register
    a = FALSE;
    spin_t_flags = NOTX;    			// zrusit priznak vysilani
    C_U0TENL;              				// zakazat preruseni
#if RS485								// pokud se jedna o simplexni komunikaci
    if(spin_flags & RXTXSW)				// pokud se jedna o rezim RS485
     C_DE;								// prepnout na prijem
#endif
    cled_count = 0;
   }
  C_U0TXI;      						// zrusit priznak interruptu
 }

/*------------------------------------------------------------------
 DMA Interrupt
-------------------------------------------------------------------*/
#pragma interrupt
void isr_DMA(void)
 {
  unsigned char volatile a = DSTAT;

  if((a & 0x02) == 0x02)	// preruseni od DMA1 tedy konce prenosu z RAM do UARTu
   {
    C_U0TXI;      			// zrusit priznak interruptu Tx
    S_U0TENL;               // povolit preruseni Tx
   }
  else
   asm("nop");
 }

/*------------------------------------------------------------------
 UART 0 RX Interrupt

 toto preruseni se vyvolava pouze v pripade chyby na seriove lince,
 prijem resi DMA

 NOPy je mozno podle potreby nahradit vhodnou obsluhou chyb
-------------------------------------------------------------------*/
#pragma interrupt
void isr_u0rx(void)
 {
  unsigned char r = U0STAT0, b = U0RXD; // pri chybe je nutne nacist i "prijaty" byte

  if(r & 0x40)				// Parity Error
   asm ("nop");

  if(r & 0x20)				// OverRun Error - nesmysl nebot to znamena HW chybu DMA
   asm ("nop");

  if(r & 0x10)				// Framing Error
   asm ("nop");

  if(r & 0x10)				// Break Detect
   asm ("nop");

 }

